@@ -7,8 +7,9 @@ import threading
 import warnings
 import weakref
 from concurrent.futures import Future, ThreadPoolExecutor
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable, Dict, Optional, overload
 
+from .compatibility import get_running_loop
 from .current_thread_executor import CurrentThreadExecutor
 from .local import Local
 
@@ -132,7 +133,7 @@ class AsyncToSync:
             self.main_event_loop = None
         else:
             try:
-                self.main_event_loop = asyncio.get_event_loop()
+                self.main_event_loop = get_running_loop()
             except RuntimeError:
                 # There's no event loop in this thread. Look for the threadlocal if
                 # we're inside SyncToAsync
@@ -151,7 +152,7 @@ class AsyncToSync:
     def __call__(self, *args, **kwargs):
         # You can't call AsyncToSync from a thread with a running event loop
         try:
-            event_loop = asyncio.get_event_loop()
+            event_loop = get_running_loop()
         except RuntimeError:
             pass
         else:
@@ -238,7 +239,11 @@ class AsyncToSync:
                     tasks = asyncio.Task.all_tasks(loop)
                 for task in tasks:
                     task.cancel()
-                loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
+
+                async def gather():
+                    await asyncio.gather(*tasks, return_exceptions=True)
+
+                loop.run_until_complete(gather())
                 for task in tasks:
                     if task.cancelled():
                         continue
@@ -320,7 +325,7 @@ class SyncToAsync:
 
     # If they've set ASGI_THREADS, update the default asyncio executor for now
     if "ASGI_THREADS" in os.environ:
-        loop = asyncio.get_event_loop()
+        loop = get_running_loop()
         loop.set_default_executor(
             ThreadPoolExecutor(max_workers=int(os.environ["ASGI_THREADS"]))
         )
@@ -343,6 +348,15 @@ class SyncToAsync:
     else:
         thread_sensitive_context: None = None
 
+    # Contextvar that is used to detect if the single thread executor
+    # would be awaited on while already being used in the same context
+    if sys.version_info >= (3, 7):
+        deadlock_context: "contextvars.ContextVar[bool]" = contextvars.ContextVar(
+            "deadlock_context"
+        )
+    else:
+        deadlock_context: None = None
+
     # Maintaining a weak reference to the context ensures that thread pools are
     # erased once the context goes out of scope. This terminates the thread pool.
     context_to_thread_executor: "weakref.WeakKeyDictionary[object, ThreadPoolExecutor]" = (
@@ -370,7 +384,7 @@ class SyncToAsync:
             pass
 
     async def __call__(self, *args, **kwargs):
-        loop = asyncio.get_event_loop()
+        loop = get_running_loop()
 
         # Work out what thread to run the code in
         if self._thread_sensitive:
@@ -391,9 +405,15 @@ class SyncToAsync:
                     # Create new thread executor in current context
                     executor = ThreadPoolExecutor(max_workers=1)
                     self.context_to_thread_executor[thread_sensitive_context] = executor
+            elif self.deadlock_context and self.deadlock_context.get(False):
+                raise RuntimeError(
+                    "Single thread executor already being used, would deadlock"
+                )
             else:
                 # Otherwise, we run it in a fixed single thread
                 executor = self.single_thread_executor
+                if self.deadlock_context:
+                    self.deadlock_context.set(True)
         else:
             # Use the passed in executor, or the loop's default if it is None
             executor = self._executor
@@ -424,6 +444,8 @@ class SyncToAsync:
 
         if contextvars is not None:
             _restore_context(context)
+        if self.deadlock_context:
+            self.deadlock_context.set(False)
 
         return ret
 
@@ -488,11 +510,29 @@ class SyncToAsync:
 async_to_sync = AsyncToSync
 
 
+@overload
 def sync_to_async(
-    func: Optional[Callable[..., Any]] = None,
+    func: None = None,
     thread_sensitive: bool = True,
     executor: Optional["ThreadPoolExecutor"] = None,
-) -> Union[SyncToAsync, Callable[[Callable[..., Any]], SyncToAsync]]:
+) -> Callable[[Callable[..., Any]], SyncToAsync]:
+    ...
+
+
+@overload
+def sync_to_async(
+    func: Callable[..., Any],
+    thread_sensitive: bool = True,
+    executor: Optional["ThreadPoolExecutor"] = None,
+) -> SyncToAsync:
+    ...
+
+
+def sync_to_async(
+    func=None,
+    thread_sensitive=True,
+    executor=None,
+):
     if func is None:
         return lambda f: SyncToAsync(
             f,
